/**
 * \file legendbutton.h
 * A button with drop-down menu and some signals
 *
 * \b Project: Qoobar
 * \author Alex Novichkov
 * \date 11 Aug 2011
 *
 * Copyright (C) 2011  Alex Novichkov
 *
 * This file is part of Qoobar.
 *
 * Qoobar is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Qoobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "legendbutton.h"
#ifdef HAVE_QT5
#include <QtWidgets>
#else
#include <QtGui>
#endif

#include "enums.h"
#include "placeholders.h"

LegendButton::LegendButton(QWidget *parent) :
    QPushButton(parent), categories(NoPlaceholders), menu(new QMenu(this))
{
    setText(tr("Legend"));

    connect(menu,SIGNAL(triggered(QAction *)),SLOT(actionTriggered(QAction *)));
    setMenu(menu);
}

bool LegendButton::event(QEvent *e)
{
    if (e->type()==QEvent::LanguageChange) {
        retranslateUi();
        return false;
    }
    return QPushButton::event(e);
}

void LegendButton::retranslateUi()
{
    setText(tr("Legend"));
    menu->clear();
    QString actions;
    if (categories & WritablePlaceholders)
        actions.append(Placeholders::getWritablePlaceholders());
    if (categories & ReadOnlyPlaceholders)
        actions.append(Placeholders::getReadOnlyPlaceholders());
    if (categories & VoidPlaceholder)
        actions.append(Placeholders::getVoidPlaceholder());
    Q_FOREACH (const QChar &c, actions) {
        QAction *a=new QAction(Placeholders::placeholderToDescription(c),menu);
        a->setData(c);
        menu->addAction(a);
    }
}

void LegendButton::actionTriggered(QAction *a)
{
    emit placeholderChosen(QString("%%1").arg(a->data().toString()));
}
